from django.contrib import admin

from model_log.mixins import ModelAdminMixin
from .models import Log


class LogAdmin(admin.ModelAdmin):
    fields = ('object_id', 'model', 'action', 'data', 'date_created', 'user')

    list_display = ('id', 'object_id', 'model', 'action', 'date_created')

    readonly_fields = ('object_id', 'model', 'action', 'data', 'date_created', 'user')

    search_fields = ('id', 'object_id', 'model', 'action', 'date_created', 'user')

    change_form_template = "change_form.html"

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False


class LogModelAdmin(ModelAdminMixin, admin.ModelAdmin):
    """ Use this class instead of ModelAdmin. """
    pass


admin.site.register(Log, LogAdmin)



from ..utils import model_to_dict, diff, create_log


def save_object(sender, instance, created, using, **kwargs):
    data = diff(instance)
    action = 'C' if created else 'U'
    create_log(instance, action, using, data)


def init_object_attrs(sender, instance, **kwargs):
    model_dict = {}

    if instance.id:
        model_dict = model_to_dict(instance)
    instance.__attrs = model_dict


def delete_object(sender, instance, using, **kwargs):
    data = {}
    action = 'D'
    create_log(instance, action, using, data)



# Generated by Django 3.1.7 on 2021-02-23 15:02

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True, db_index=True, help_text='The date and time this changes was.', verbose_name='Date created')),
                ('object_id', models.IntegerField(help_text='Primary key of the object under version control.')),
                ('object', models.CharField(help_text='Model of the object under version control.', max_length=128)),
                ('data', models.TextField(blank=True, help_text='The data being changed.', null=True)),
                ('action', models.CharField(choices=[('C', 'Created'), ('U', 'Updated'), ('D', 'Deleted')], help_text='created|updated|deleted', max_length=1, verbose_name='Action')),
                ('user', models.ForeignKey(blank=True, help_text='The user who created this changes.', null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-pk',),
            },
        ),
    ]



from django.contrib.admin.utils import unquote
from django.core.exceptions import PermissionDenied
from django.template.response import TemplateResponse
from django.utils.text import capfirst
from django.utils.translation import gettext as _
from model_log.models import Log


class ModelAdminMixin:
    object_history_template = "object_history.html"

    def history_view(self, request, object_id, extra_context=None):
        "The 'history' admin view for this model."
        from django.contrib.admin.models import LogEntry
        # First check if the user can see this history.
        model = self.model
        obj = self.get_object(request, unquote(object_id))
        if obj is None:
            return self._get_obj_does_not_exist_redirect(request, model._meta, object_id)

        if not self.has_view_or_change_permission(request, obj):
            raise PermissionDenied

        # Then get the history for this object.
        opts = model._meta
        app_label = opts.app_label
        log_data = Log.objects.filter(model=obj.__class__.__name__.lower())

        context = {
            **self.admin_site.each_context(request),
            'title': _('Change history: %s') % obj,
            'action_list': log_data,
            'module_name': str(capfirst(opts.verbose_name_plural)),
            'object': obj,
            'opts': opts,
            'preserved_filters': self.get_preserved_filters(request),
            **(extra_context or {}),
        }

        request.current_app = self.admin_site.name

        return TemplateResponse(request, self.object_history_template or [
            "admin/%s/%s/object_history.html" % (app_label, opts.model_name),
            "admin/%s/object_history.html" % app_label,
            "admin/object_history.html"
        ], context)



# Generated by Django 3.1.7 on 2021-02-23 18:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('model_log', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='log',
            old_name='object',
            new_name='model',
        ),
        migrations.RemoveField(
            model_name='log',
            name='user',
        ),
    ]



# Generated by Django 3.1.7 on 2021-02-24 14:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('model_log', '0002_auto_20210223_1345'),
    ]

    operations = [
        migrations.AddField(
            model_name='log',
            name='user',
            field=models.TextField(blank=True, help_text='ID of the user who makes the action', null=True),
        ),
    ]



from setuptools import setup

files = ["migrations/*", "signals/*", "templates/*"]

setup(
  name='django-model-logs',        
  packages=['model_log'],  
  package_data={'model_log': files},
  version='0.1.11',
  description='Library for logging changes of each model',
  long_description=open('README.rst').read(),
  long_description_content_type='text/x-rst',
  author="Juan Benitez",
  license="MIT",        
  author_email='juanbenitezdev@gmail.com',
  url='https://github.com/JuanBenitezDev/django-model-logs',
  install_requires=['Django>=2.2'],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',     
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',
  ],
)



from django.db import models


class Log(models.Model):
    ACTIONS = (
        ('C', "Created"),
        ('U', "Updated"),
        ('D', "Deleted")
    )

    date_created = models.DateTimeField("Date created", db_index=True, auto_now_add=True,
                                        help_text="The date and time this changes was.")
    object_id = models.IntegerField(help_text="Primary key of the object under version control.")
    model = models.CharField(help_text="Model of the object under version control.", max_length=128)
    data = models.TextField(blank=True, null=True, help_text="The data being changed.")
    action = models.CharField("Action", choices=ACTIONS, help_text='created|updated|deleted', max_length=1)
    user = models.TextField(blank=True, null=True, help_text='ID of the user who makes the action')

    def __str__(self):
        text = "Changes on {} of {} at {}".format(
            self.object_id,
            self.model,
            self.date_created.strftime('%Y-%m-%d %H:%M'))
        return text

    @property
    def action_label(self):
        return dict(self.ACTIONS)[self.action]

    class Meta:
        ordering = ("-pk",)
